@@ -1,5 +1,7 @@
+from __future__ import (absolute_import, print_function, division)
 import threading, Queue, cStringIO
-import tcp, certutils
+import OpenSSL
+from . import tcp, certutils
 
 class ServerThread(threading.Thread):
     def __init__(self, server):
@@ -16,17 +18,18 @@ class ServerThread(threading.Thread):
 class ServerTestBase:
     ssl = None
     handler = None
+    addr = ("localhost", 0)
     @classmethod
     def setupAll(cls):
         cls.q = Queue.Queue()
         s = cls.makeserver()
-        cls.port = s.port
+        cls.port = s.address.port
         cls.server = ServerThread(s)
         cls.server.start()
 
     @classmethod
     def makeserver(cls):
-        return TServer(cls.ssl, cls.q, cls.handler)
+        return TServer(cls.ssl, cls.q, cls.handler, cls.addr)
 
     @classmethod
     def teardownAll(cls):
@@ -38,40 +41,46 @@ class ServerTestBase:
 
 
 class TServer(tcp.TCPServer):
-    def __init__(self, ssl, q, handler_klass, addr=("127.0.0.1", 0)):
+    def __init__(self, ssl, q, handler_klass, addr):
         """
-            ssl: A {cert, key, v3_only} dict.
+            ssl: A dictionary of SSL parameters:
+                
+                    cert, key, request_client_cert, cipher_list,
+                    dhparams, v3_only
         """
-        tcp.TCPServer.__init__(self, addr)
+        tcp.TCPServer.__init__(self, addr, use_ipv6=use_ipv6)
         self.ssl, self.q = ssl, q
         self.handler_klass = handler_klass
         self.last_handler = None
 
-    def handle_connection(self, request, client_address):
+    def handle_client_connection(self, request, client_address):
         h = self.handler_klass(request, client_address, self)
         self.last_handler = h
         if self.ssl:
             cert = certutils.SSLCert.from_pem(
-                file(self.ssl["cert"], "r").read()
+                file(self.ssl["cert"], "rb").read()
             )
+            raw = file(self.ssl["key"], "rb").read()
+            key = OpenSSL.crypto.load_privatekey(OpenSSL.crypto.FILETYPE_PEM, raw)
             if self.ssl["v3_only"]:
                 method = tcp.SSLv3_METHOD
-                options = tcp.OP_NO_SSLv2|tcp.OP_NO_TLSv1
+                options = OpenSSL.SSL.OP_NO_SSLv2 | OpenSSL.SSL.OP_NO_TLSv1
             else:
                 method = tcp.SSLv23_METHOD
                 options = None
             h.convert_to_ssl(
-                cert,
-                self.ssl["key"],
+                cert, key,
                 method = method,
                 options = options,
                 handle_sni = getattr(h, "handle_sni", None),
-                request_client_cert = self.ssl["request_client_cert"]
+                request_client_cert = self.ssl["request_client_cert"],
+                cipher_list = self.ssl.get("cipher_list", None),
+                dhparams = self.ssl.get("dhparams", None)
             )
         h.handle()
         h.finish()
 
-    def handle_error(self, request, client_address):
+    def handle_error(self, connection, client_address, fp=None):
         s = cStringIO.StringIO()
-        tcp.TCPServer.handle_error(self, request, client_address, s)
+        tcp.TCPServer.handle_error(self, connection, client_address, s)
         self.q.put(s.getvalue())
